<?php

function sankloud_theme_options_init() {

	// If we have no options in the database, let's add them now.
	if ( false === sankloud_get_theme_options() )
		add_option( 'sankloud_theme_options', sankloud_get_default_theme_options() );

	register_setting(
		'sankloud_options',       // Options group, see settings_fields() call in theme_options_render_page()
		'sankloud_theme_options', // Database option, see sankloud_get_theme_options()
		'sankloud_theme_options_validate' // The sanitization callback, see sankloud_theme_options_validate()
	);
}
add_action( 'admin_init', 'sankloud_theme_options_init' );

/**
 * Change the capability required to save the 'sankloud_options' options group.
 *
 * @see sankloud_theme_options_init() First parameter to register_setting() is the name of the options group.
 * @see sankloud_theme_options_add_page() The edit_theme_options capability is used for viewing the page.
 *
 * By default, the options groups for all registered settings require the manage_options capability.
 * This filter is required to change our theme options page to edit_theme_options instead.
 * By default, only administrators have either of these capabilities, but the desire here is
 * to allow for finer-grained control for roles and users.
 *
 * @param string $capability The capability used for the page, which is manage_options by default.
 * @return string The capability to actually use.
 */
function sankloud_option_page_capability( $capability ) {
	return 'edit_theme_options';
}
add_filter( 'option_page_capability_sankloud_options', 'sankloud_option_page_capability' );

/**
 * Add our theme options page to the admin menu, including some help documentation.
 *
 * This function is attached to the admin_menu action hook.
 *
 * @since sankloud 1.0
 */
function sankloud_theme_options_add_page() {
	$theme_page = add_theme_page(
		__( 'Theme Options', 'San Kloud' ), // Name of page
		__( 'Theme Options', 'San Kloud' ), // Label in menu
		'edit_theme_options',                  // Capability required
		'theme_options',                       // Menu slug, used to uniquely identify the page
		'theme_options_render_page'            // Function that renders the options page
	);

	if ( ! $theme_page )
		return;

	$help = '<p>' . __( 'Some themes provide customization options that are grouped together on a Theme Options screen. If you change themes, options may change or disappear, as they are theme-specific. Your current theme, sankloud, provides the following Theme Options:', 'sankloud' ) . '</p>' .
			'<ol>' .
				'<li>' . __( '<strong>Color Scheme</strong>: You can choose a color palette of "Light" (light background with dark text) or "Dark" (dark background with light text) for your site.', 'sankloud' ) . '</li>' .
			'</ol>' .
			'<p>' . __( 'Remember to click "Save Changes" to save any changes you have made to the theme options.', 'sankloud' ) . '</p>';

	add_contextual_help( $theme_page, $help );
}
add_action( 'admin_menu', 'sankloud_theme_options_add_page' );

/**
 * Returns an array of color schemes registered for sankloud.
 *
 * @since sankloud 1.0
 */
function sankloud_color_schemes() {
	$color_scheme_options = array(
		'Default' => array(
			'value' => 'light',
			'label' => __( 'Light', 'sankloud' ),
			'thumbnail' => get_template_directory_uri() . '/inc/images/light.png',
			'default_link_color' => '#1b8be0',
		)
	);

	return apply_filters( 'sankloud_color_schemes', $color_scheme_options );
}

/**
 * Returns the default options for sankloud.
 *
 * @since sankloud 1.0
 */
function sankloud_get_default_theme_options() {
	$default_theme_options = array(
		'color_scheme' => 'default',
		'link_color'   => sankloud_get_default_link_color( 'default' ),
		'theme_layout' => 'content-sidebar',
	);

	if ( is_rtl() )
 		$default_theme_options['theme_layout'] = 'sidebar-content';

	return apply_filters( 'sankloud_default_theme_options', $default_theme_options );
}

/**
 * Returns the default link color for sankloud, based on color scheme.
 *
 * @since sankloud 1.0
 *
 * @param $string $color_scheme Color scheme. Defaults to the active color scheme.
 * @return $string Color.
*/
function sankloud_get_default_link_color( $color_scheme = null ) {
	if ( null === $color_scheme ) {
		$options = sankloud_get_theme_options();
		$color_scheme = $options['color_scheme'];
	}

	$color_schemes = sankloud_color_schemes();
	if ( ! isset( $color_schemes[ $color_scheme ] ) )
		return false;

	return $color_schemes[ $color_scheme ]['default_link_color'];
}

/**
 * Returns the options array for sankloud.
 *
 * @since sankloud 1.0
 */
function sankloud_get_theme_options() {
	return get_option( 'sankloud_theme_options', sankloud_get_default_theme_options() );
}

/**
 * Returns the options array for sankloud.
 *
 * @since sankloud 1.0
 */
function theme_options_render_page() {
	?>
	<div class="wrap">
		<?php screen_icon(); ?>
		<h2><?php printf( __( '%s Theme Options', 'sankloud' ), get_current_theme() ); ?></h2>
		<?php settings_errors(); ?>

		<form method="post" action="options.php">
			<?php
				settings_fields( 'sankloud_options' );
				$options = sankloud_get_theme_options();
				$default_options = sankloud_get_default_theme_options();
			?>

			<table class="form-table">

				<tr valign="top" class="image-radio-option color-scheme"><th scope="row"><?php _e( 'Color Scheme', 'sankloud' ); ?></th>
					<td>
						<fieldset><legend class="screen-reader-text"><span><?php _e( 'Color Scheme', 'sankloud' ); ?></span></legend>
						<?php
							foreach ( sankloud_color_schemes() as $scheme ) {
								?>
								<div class="layout">
								<label class="description">
									<input type="radio" name="sankloud_theme_options[color_scheme]" value="<?php echo esc_attr( $scheme['value'] ); ?>" <?php checked( $options['color_scheme'], $scheme['value'] ); ?> />
									<input type="hidden" id="default-color-<?php echo esc_attr( $scheme['value'] ); ?>" value="<?php echo esc_attr( $scheme['default_link_color'] ); ?>" />
									<span>
										<img src="<?php echo esc_url( $scheme['thumbnail'] ); ?>" width="136" height="122" alt="" />
										<?php echo $scheme['label']; ?>
									</span>
								</label>
								</div>
								<?php
							}
						?>
						</fieldset>
					</td>
				</tr>
			</table>

			<?php submit_button(); ?>
		</form>
	</div>
	<?php
}

/**
 * Sanitize and validate form input. Accepts an array, return a sanitized array.
 *
 * @see sankloud_theme_options_init()
 * @todo set up Reset Options action
 *
 * @since sankloud 1.0
 */
function sankloud_theme_options_validate( $input ) {
	$output = $defaults = sankloud_get_default_theme_options();

	// Color scheme must be in our array of color scheme options
	if ( isset( $input['color_scheme'] ) && array_key_exists( $input['color_scheme'], sankloud_color_schemes() ) )
		$output['color_scheme'] = $input['color_scheme'];

	return apply_filters( 'sankloud_theme_options_validate', $output, $input, $defaults );
}

/**
 * Enqueue the styles for the current color scheme.
 *
 * @since sankloud 1.0
 */
function sankloud_enqueue_color_scheme() {
	$options = sankloud_get_theme_options();
	$color_scheme = $options['color_scheme'];

	if ( 'dark' == $color_scheme )
		wp_enqueue_style( 'dark', get_template_directory_uri() . '/colors/dark.css', array(), null );

	do_action( 'sankloud_enqueue_color_scheme', $color_scheme );
}
add_action( 'wp_enqueue_scripts', 'sankloud_enqueue_color_scheme' );

/**
 * Add a style block to the theme for the current link color.
 *
 * This function is attached to the wp_head action hook.
 *
 * @since sankloud 1.0
 */
function sankloud_print_link_color_style() {
	$options = sankloud_get_theme_options();
	$link_color = $options['link_color'];

	$default_options = sankloud_get_default_theme_options();

	// Don't do anything if the current link color is the default.
	if ( $default_options['link_color'] == $link_color )
		return;
?>
	<style>
		/* Link color */
		a,
		#site-title a:focus,
		#site-title a:hover,
		#site-title a:active,
		.entry-title a:hover,
		.entry-title a:focus,
		.entry-title a:active,
		.widget_sankloud_ephemera .comments-link a:hover,
		section.recent-posts .other-recent-posts a[rel="bookmark"]:hover,
		section.recent-posts .other-recent-posts .comments-link a:hover,
		.format-image footer.entry-meta a:hover,
		#site-generator a:hover {
			color: <?php echo $link_color; ?>;
		}
		section.recent-posts .other-recent-posts .comments-link a:hover {
			border-color: <?php echo $link_color; ?>;
		}
		article.feature-image.small .entry-summary p a:hover,
		.entry-header .comments-link a:hover,
		.entry-header .comments-link a:focus,
		.entry-header .comments-link a:active,
		.feature-slider a.active {
			background-color: <?php echo $link_color; ?>;
		}
	</style>
<?php
}
add_action( 'wp_head', 'sankloud_print_link_color_style' );
?>